package tax;

import java.io.*;

/**
 * An encapsulation of a tax table.
 * 
 * @author Prof. David Bernstein, James Madison University
 *
 */
public class TaxTable
{
  private double[] boundaries, rates;
  private String filingStatus;
  
  /**
   * Default Constructor.
   */
  public TaxTable()
  {
  }
  
  /**
   * Calculate the base tax (i.e., the tax for the brackets before the marginal bracket.
   * 
   * @param income  The income
   * @return        The tax
   */
  private double calculateBaseTax(double income)
  {
    int top = indexOf(income);
    double total = 0.0;
    for (int i=1; i<top; i++)
    {
      if (i == 1)total += rates[i]/100.0 * (boundaries[i] - 0.00);
      else       total += rates[i]/100.0 * (boundaries[i] - boundaries[i-1]);
    }
    return total;
  }

  /**
   * Get the filing status associated with this TaxTable.
   * 
   * @return The filing status (or null if undefined)
   */
  public String getFilingStatus()
  {
    return filingStatus;
  }
  
  /**
   * Get the marginal tax rate when it is interpolated.
   * 
   * @param income  The income
   * @return        The tax rate
   */
  public double getInterpolatedRate(double income)
  {
    int index = indexOf(income);
    if (index == rates.length-1) return rates[index];

    double lambda = (income - boundaries[index-1]) / (boundaries[index] - boundaries[index-1]);

    return lambda*rates[index] + (1.0 - lambda)*rates[index-1];
  }

  /**
   * Get the tax when the marginal tax rate is interpolated.
   * 
   * @param income  The income
   * @return        The tax
   */
  public double getInterpolatedTax(double income)
  {
    int top = indexOf(income);
    if (top == 1) 
      return calculateBaseTax(income) 
          + getInterpolatedRate(income)/100.0 * income;
    else          
      return calculateBaseTax(income) 
          + getInterpolatedRate(income)/100.0 * (income - (boundaries[top-1]));
  }

  /**
   * Get the marginal tax rate when it isn't interpolated.
   * 
   * @param income  The income
   * @return        The tax rate
   */
  public double getUninterpolatedRate(double income)
  {
    return rates[indexOf(income)];
  }

  /**
   * Get the tax when the marginal tax rate isn't interpolated.
   * 
   * @param income  The income
   * @return        The tax
   */
  public double getUninterpolatedTax(double income)
  {
    int top = indexOf(income);
    if (top == 1) 
      return calculateBaseTax(income) 
          + getUninterpolatedRate(income)/100.0 * income;
    else          
      return calculateBaseTax(income) 
          + getUninterpolatedRate(income)/100.0 * (income - (boundaries[top-1]));
  }

  /**
   * Get the bracket-index for a particular income level.
   * 
   * @param income   The income
   * @return The bracket index
   */
  private int indexOf(double income)
  {
    int i = 0;
    while ((i < boundaries.length) && (income > boundaries[i])) ++i;

    return i;
  }

  /**
   * Read the attributes of this TaxTable.
   * 
   * @param fileName  The name of the file to read from
   * @throws IOException  If something goes wrong
   */
  public void read(String fileName) throws IOException
  {
    BufferedReader in = new BufferedReader(new FileReader(fileName + ".tax"));
    String line = in.readLine();
    String[] tokens = line.split("\t");
    filingStatus = tokens[0];
    int size = Integer.parseInt(tokens[1]);
    
    boundaries = new double[size];
    rates     = new double[size];
    for (int i=0; i<size; i++)
    {
      line = in.readLine();
      tokens = line.split("\t");

      rates[i] = Double.parseDouble(tokens[0]);

      if (i == size-1) boundaries[i] = Double.POSITIVE_INFINITY;
      else             boundaries[i] = Double.parseDouble(tokens[1]);
    }
    in.close();
  }

  /**
   * Set the boundaries of the brackets.
   * 
   * @param boundaries  The boundaries
   */
  public void setBoundaries(double[] boundaries)
  {
    this.boundaries = boundaries;
  }

  /**
   * Set the rates for the brackets.
   * 
   * @param rates  The rates
   */
  public void setRates(double[] rates)
  {
    this.rates = rates;
  }
}
