package taxeze.gui;


import static taxeze.gui.TaxezeWindow.*;

import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;

import tax.*;

/**
 * The observer of all GUI components in the TaxezeWindow.
 * 
 * @author Prof. David Bernstein, James Madison University
 */
public class TaxezeController implements ActionListener, DocumentListener, ItemListener
{
  static final String DATA_FILES         = "Data Files";
  static final String ERROR              = "Error";
  static final String ERROR_OPENING_FILE = "Error Opening File";
  static final String INVALID_FILE_TYPE  = "Invalid File Type";
  static final String NO_MATCHES         = "No Matches";
  
  private FileNameExtensionFilter taxFilter;
  private TaxezeWindow            window;
  private JFileChooser            fileChooser;


  private TaxSchedule schedule;
  
  /**
   * Default Constructor.
   */
  public TaxezeController()
  {
    super();
    taxFilter = new FileNameExtensionFilter(DATA_FILES, "tax");
    fileChooser = new JFileChooser();
    fileChooser.setCurrentDirectory(new File("."));
    
    schedule = new TaxSchedule("United States");
  }
  
  /**
   * Handle actionPerformed() message (required by ActionListener).
   * 
   * @param ae   The ActionEvent that generated the message
   */
  public void actionPerformed(ActionEvent ae)
  {
    String ac = ae.getActionCommand();
    
    if (ac.equals(CALCULATE))   calculate();
    else if (ac.equals(EXIT))   System.exit(0);
    else if (ac.equals(READ))   read();
  }
  
  /**
   * Handle a CALCULATE request.
   */
  private void calculate()
  {
    List<String>  results = new ArrayList<String>();
    window.displayResults(null);
    
    TaxTable table = schedule.get(window.getFilingStatus());
    if (table == null)
    {
      results.add("Invalid filing status.");
    }
    else
    {
      double income = window.getTaxableIncome();
      if (income < 0.00) income = 0.00;
      double marginalRate = table.getUninterpolatedRate(income);
      double taxes = table.getUninterpolatedTax(income);
      results.add(String.format("Marginal Tax Rate: %%%5.2f", marginalRate));
      results.add(String.format("Tax: $%10.2f", taxes));
    }
    
    window.displayResults(results);

  }
  
  /**
   * Handle changedUpdate messages.
   * 
   * @param de The event that generated the message
   */
  @Override
  public void changedUpdate(DocumentEvent e)
  {
    // Shouldn't happen
  }
  
  /**
   * Determine if calculations are possible.
   */
  private void checkCalculatability()
  {
    boolean disable = window.filingStatusField.getSelectedItem() == null
        || window.filingStatusField.getSelectedItem().equals("")
        || window.incomeField.getText().equals("") || window.incomeField.getText() == null;
    
    window.enableCalculation(!disable);
  }
  

  /**
   * Handle insertUpdate messages.
   * 
   * @param de The event that generated the message
   */
  @Override
  public void insertUpdate(DocumentEvent de)
  {
    checkCalculatability();
  }
  
  /**
   * Handle itemStateChanged message.
   * 
   * @param ie The event that generated the message
   */
  @Override
  public void itemStateChanged(ItemEvent ie)
  {
    checkCalculatability();
  }
  
  /**
   * Handle a READ request.
   */
  private void read()
  {
    fileChooser.setFileFilter(taxFilter);
    int result = fileChooser.showOpenDialog(null);
    if (result == JFileChooser.APPROVE_OPTION) // Read a tax table
    {
      String path = fileChooser.getSelectedFile().getPath();
      String extension = taxFilter.getExtensions()[0];
      if (path.endsWith(extension))
      {
        String name = path.substring(0, path.length() - 4);
        try
        {
          TaxTable table = new TaxTable();
          table.read(name);
          String filingStatus = table.getFilingStatus();   
          schedule.put(filingStatus, table);
          
          setTaxScheduleIsAvailable(true);
          window.addFilingStatus(filingStatus);
        }
        catch (IOException ioe)
        {
          JOptionPane.showMessageDialog(null, ERROR_OPENING_FILE, ERROR, JOptionPane.ERROR_MESSAGE);
        }
      }
      else
      {
        JOptionPane.showMessageDialog(null, INVALID_FILE_TYPE, ERROR, JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  /**
   * Handle removeUpdate messages.
   * 
   * @param de The event that generated the message
   */
  @Override
  public void removeUpdate(DocumentEvent e)
  {
    checkCalculatability();
  } 
  
  /**
   * Set whether a TaxSchedule is available to use or not.
   * 
   * @param boolean true to indicate a TaxTable is available; false otherwise
   */
  private void setTaxScheduleIsAvailable(boolean available)
  {
    window.enableDataEntry(available);
  }
  
  /**
   * Set the TaxezeWindow that this object is controlling.
   * 
   * @param window  The window.
   */
  public void setWindow(TaxezeWindow window)
  {
    this.window = window;
  }
}
